/*
  TextEdit.h 

  Will Eighteen, 2000: will@opaline.demon.co.uk             

  This program is free software; you can redistribute it and/or modify       
  it under the terms of the GNU General Public License as published by       
  the Free Software Foundation; either version 2 of the License, or          
  (at your option) any later version.                                        

  This program is distributed in the hope that it will be useful,            
  but WITHOUT ANY WARRANTY; without even the implied warranty of             
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              
  GNU General Public License for more details.                               

  You should have received a copy of the GNU General Public License          
  along with this program; if not, write to the Free Software                
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                  

  See COPYING for full details.                                  
*/


#ifndef TEXTEDIT_H
#define TEXTEDIT_H

#include "NoUndoCmd.h"
#include <Xm/Xm.h>


typedef void ( *TextCallback ) ( void * , char * );
typedef void ( *TextCancelledCallback ) ( void * );


class TextEdit : public NoUndoCmd
{
	public:
		TextEdit ( char *, int, TextCallback, TextCancelledCallback, void * );
		void setColumns ( int );
		void setRows ( int );
		void setText ( char * );
		void setHelp ( char *text ) { _helpText = text; }
		void setEditStatus ( Boolean status );
		void close();
		
	protected:
		void doCommand();		// called by base class
		TextCallback _callback;	// function to be called
		TextCancelledCallback _cancelCallback;
		void *_clientData;		// data provided by caller
		
	private:
		// callback for WM 'X'
		static void clientMessageProc ( Widget, XtPointer, XtPointer );

		static void okCallback ( Widget, XtPointer, XtPointer );
		static void cancelCallback ( Widget, XtPointer, XtPointer );
		static void clearCallback ( Widget, XtPointer, XtPointer );
		static void helpCallback ( Widget, XtPointer, XtPointer );
		static void refreshCallback ( Widget, XtPointer, XtPointer );
		void clientMessage ( XmAnyCallbackStruct * );
		void ok ( char * );
		void cancel();
		void clear();
		void help( void * );
		char _name[128];
		char _applicationName[128];
		void *_helpText;
		void *_data;	// text entered or displayed in widget
		void *_copy;
		int _presetText;
		int _columns;
		int _rows;
		Boolean _editable;
		Widget _text;
		Widget _popup, _rc1, _rc2;
		Widget _ok, _cancel, _clear, _help, _refresh;
		Widget _label;
};

#endif


/*
  Notes:
  
  Selecting text by highlighting with the mouse or SHIFT+<arrow key> results
  in the highlighted text being returned when 'ok' is pressed.
  
  close() calls cancel, returning selected item number 0.
  
  
  Usage:
  	call setRows(), setColumns(), setHelp() (optional). execute(), setText().
	
	
  Bugs:
    widget destruction enabled in clear , refresh and help callbacks removes buttons;
   as things are, they aren't destroyed, possibly leading to memory leakage.
*/
